﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using GeocodingApi;

namespace GenerateKML
{
    /// <summary>
    /// This class is used to cenerate the kml file
    /// </summary>
    public class Generator
    {
        private static readonly int DelayInMs = 500;

        private static readonly List<string> Colors = new List<string>
        {
            "#red",
            "#green",
            "#blue",
            "#yellow",
        };

        private kml _Kml;

        private int _ColorIndex = 0;

        private string _LastErrorOrWarning = null;

        /// <summary>
        /// The last error or warning 
        /// </summary>
        public string LastErrorOrWarning
        {
            get
            {
                return _LastErrorOrWarning;
            }
        }

        /// <summary>
        /// Constractor
        /// </summary>
        /// <param name="kmlName">name of the kml</param>
        public Generator(string kmlName)
        {
            _Kml = new kml(kmlName);
        }

        /// <summary>
        /// Generate placemark by address description
        /// </summary>
        /// <param name="addrDescription">address and description</param>
        /// <returns>if no matched, return false</returns>
        public bool Generate(AddressDescription addrDescription)
        {
            _LastErrorOrWarning = null;

            Thread.Sleep(DelayInMs);

            List<GeographicCoordinate> coordinates = Geocoding.Geocode(addrDescription.Address);

            if (coordinates.Count == 0)
            {
                _LastErrorOrWarning = string.Format("Address:{0}, Description:{1} does not find the coordinates, please make sure the address is correctly.",
                    addrDescription.Address, addrDescription.Description);

                return false;
            }

            if (coordinates.Count > 1)
            {
                _LastErrorOrWarning = string.Format("Address:{0}, Description:{1} has more than one coordinates.",
                    addrDescription.Address, addrDescription.Description);
            }

            string colorId = Colors[_ColorIndex];

            _ColorIndex++;

            if (_ColorIndex > Colors.Count)
            {
                _ColorIndex = 0;
            }

            _Kml.Document.Add(new Placemark(addrDescription.Address, addrDescription.Description, colorId,
                coordinates[0].Latitude, coordinates[0].Longitude));

            return true;
        }

        /// <summary>
        /// Save kml file
        /// </summary>
        /// <param name="filePath">file Path of the kml</param>
        public void Save(string filePath)
        {
            _Kml.SaveToFile(filePath);
        }
    }
}
