﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;

namespace GenerateKML
{
    public class Placemark
    {
        public class KMLPoint
        {
            public KMLPoint()
            {
            }

            public KMLPoint(double latitude, double longitude)
            {
                SetCoordinates(latitude, longitude);
            }

            private string _coordinates;

            public void SetCoordinates(double latitude, double longitude)
            {
                _coordinates = longitude.ToString() + "," + latitude.ToString();
            }

            public string coordinates
            {
                get
                {
                    return _coordinates;
                }

                set
                {
                    _coordinates = value;
                }
            }
        }

        [XmlElement("name")]
        public string Name { get; set; }

        [XmlElement("description")]
        public string Description { get; set; }

        [XmlElement("styleUrl")]
        public string StyleUrl { get; set; }

        public KMLPoint Point { get; set; }

        public Placemark()
        {
        }

        public Placemark(string name, string description, string styleUrl,
            double latitude, double longitude)
        {
            Name = name;
            Description = description;
            StyleUrl = styleUrl;

            Point = new KMLPoint(latitude, longitude);
        }

    }

    public class kml
    {
        [XmlIgnore]
        string Name { get; set; }


        List<Placemark> _Placemarks = new List<Placemark>();

        [XmlArray()]
        public List<Placemark> Document
        {
            get
            {
                return _Placemarks;
            }

            set
            {
                _Placemarks = value;
            }
        }

        public kml()
        {
        }

        public kml(string name)
        {
            Name = name;
        }

        private XmlNode GetColorStyle(XmlDocument xmlDoc,  string color)
        {
            XmlNode style = xmlDoc.CreateNode(XmlNodeType.Element, "Style", "");
            XmlAttribute attr = style.OwnerDocument.CreateAttribute("id");
            attr.Value = color;
            style.Attributes.Append(attr);

            XmlNode iconStyle = xmlDoc.CreateNode(XmlNodeType.Element, "IconStyle", "");
            XmlNode icon = xmlDoc.CreateNode(XmlNodeType.Element, "Icon", "");
            XmlNode href = xmlDoc.CreateNode(XmlNodeType.Element, "href", "");
            href.InnerText = string.Format("http://www.google.com/intl/en_us/mapfiles/ms/icons/{0}-dot.png",
                color);
            
            
            style.AppendChild(iconStyle);
            iconStyle.AppendChild(icon);
            icon.AppendChild(href);

            return style;
        }


        public void SaveToFile(string xml)
        {
            using (FileStream fs = new FileStream(xml, FileMode.Create, FileAccess.ReadWrite))
            {
                using (StreamWriter sw = new StreamWriter(fs, System.Text.Encoding.UTF8))
                {
                    XmlSerializer serializer = new XmlSerializer(this.GetType());
                    serializer.Serialize(sw, this);
                }
            }


            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(xml);
            xmlDoc.CreateXmlDeclaration("1.0", "utf-8", null);

            XmlNode documentNode = xmlDoc.SelectSingleNode(@"/kml/Document");

            XmlNode nameNode = xmlDoc.CreateNode(XmlNodeType.Element, "name", "");
            nameNode.InnerText = this.Name;

            XmlNode placeMarkNode = documentNode.FirstChild;
            documentNode.InsertBefore(nameNode, placeMarkNode);

            documentNode.InsertBefore(GetColorStyle(xmlDoc, "red"), placeMarkNode);
            documentNode.InsertBefore(GetColorStyle(xmlDoc, "green"), placeMarkNode);
            documentNode.InsertBefore(GetColorStyle(xmlDoc, "blue"), placeMarkNode);
            documentNode.InsertBefore(GetColorStyle(xmlDoc, "yellow"), placeMarkNode);

            XmlNode kmlNode = xmlDoc.SelectSingleNode(@"/kml");

            XmlAttribute attr = kmlNode.OwnerDocument.CreateAttribute("xmlns");
            attr.Value = "http://earth.google.com/kml/2.0";

            kmlNode.Attributes.Append(attr);

            xmlDoc.Save(xml);
        }
    }

}
